@@ -1,9 +1,9 @@
 /*
- *  Copyright (c) 2001 Mark Fullmer and The Ohio State University
  *  All rights reserved.
-
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
+ *  Copyright (c) 2001 Mark Fullmer and The Ohio State University
  *  All rights reserved.
  *  
  *  Redistribution and use in source and binary forms, with or without 
@@ -14,7 +14,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *	   this list of conditions and the following disclaimer in the documentation 
  *	   and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *	   used to endorse or promote products derived from this software without 
  *	   specific prior written permission.
  *  
@@ -93,7 +93,9 @@ extension_info_t *GenExtensionMap(struct ftio *ftio);
 
 int flows2nfdump(struct ftio *ftio, extension_info_t *extension_info, int extended, uint32_t limitflows);
 
+#define NEED_PACKRECORD
 #include "nffile_inline.c"
+#undef NEED_PACKRECORD
 
 void usage(char *name) {
 		printf("usage %s [options] \n"
@@ -123,7 +125,7 @@ int	i;
 		return NULL;
 	}
 
-	InitExtensionMaps(NULL);
+	InitExtensionMaps(NO_EXTENSION_LIST);
 	extension_info = (extension_info_t *)malloc(sizeof(extension_info_t));
 	if ( !extension_info  ) {
 		fprintf(stderr, "malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
@@ -166,7 +168,7 @@ int	i;
 	}
 
 	extension_info->map->ex_id[i++] = 0;
-	extension_info->map->size 		= sizeof(extension_map_t) + i * sizeof(uint16_t);
+	extension_info->map->size       = sizeof(extension_map_t) + i * sizeof(uint16_t);
 
 	// align 32bits
 	if (( extension_info->map->size & 0x3 ) != 0 ) {
