@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +12,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -89,7 +90,10 @@ FilterEngine_data_t	*Engine;
 
 extern char	*FilterFilename;
 extern uint32_t loopcnt;
+
+#ifdef COMPAT15
 extern extension_descriptor_t extension_descriptor[];
+#endif
 
 /* Local Variables */
 const char *nfdump_version = VERSION;
@@ -106,7 +110,7 @@ int hash_hit = 0;
 int hash_miss = 0;
 int hash_skip = 0;
 
-extension_map_list_t extension_map_list;
+extension_map_list_t *extension_map_list;
 
 extern generic_exporter_t **exporter_list;
 /*
@@ -182,6 +186,16 @@ extern generic_exporter_t **exporter_list;
 
 #define FORMAT_bilong "%ts %td %pr %sap <-> %dap %flg %tos %opkt %ipkt %obyt %ibyt %fl"
 
+#define FORMAT_nsel "%ts %evt %xevt %pr %sap -> %dap %xsap -> %xdap %ibyt %obyt"
+
+#define FORMAT_nel "%ts %nevt %pr %sap -> %dap %nsap -> %ndap"
+
+#ifdef NSEL
+#	define DefaultMode "nsel"
+#else 
+#	define DefaultMode "line"
+#endif
+
 /* The appropriate header line is compiled automatically.
  *
  * For each defined output format a v6 long format automatically exists as well e.g.
@@ -213,14 +227,18 @@ printmap_t printmap[] = {
 	{ "bilong", 	format_special,      		FORMAT_bilong 	},
 	{ "pipe", 		flow_record_to_pipe,      	NULL 			},
 	{ "csv", 		flow_record_to_csv,      	NULL 			},
+	{ "null", 		flow_record_to_null,      	NULL 			},
+#ifdef NSEL
+	{ "nsel",		format_special, 			FORMAT_nsel		},
+	{ "nel",		format_special, 			FORMAT_nel		},
+#endif
+
 // add your formats here
 
 // This is always the last line
 	{ NULL,			NULL,                       NULL			}
 };
 
-#define DefaultMode "line"
-
 // For automatic output format generation in case of custom aggregation
 #define AggrPrependFmt	"%ts %td "
 #define AggrAppendFmt	"%pkt %byt %bps %bpp %fl"
@@ -298,7 +316,8 @@ static void usage(char *name) {
 static void PrintSummary(stat_record_t *stat_record, int plain_numbers, int csv_output) {
 static double	duration;
 uint64_t	bps, pps, bpp;
-char 		byte_str[32], packet_str[32], bps_str[32], pps_str[32], bpp_str[32];
+char 		byte_str[NUMBER_STRING_SIZE], packet_str[NUMBER_STRING_SIZE];
+char 		bps_str[NUMBER_STRING_SIZE], pps_str[NUMBER_STRING_SIZE], bpp_str[NUMBER_STRING_SIZE];
 
 	bps = pps = bpp = 0;
 	if ( stat_record->last_seen ) {
@@ -326,11 +345,11 @@ char 		byte_str[32], packet_str[32], bps_str[32], pps_str[32], bpp_str[32];
 			(long long unsigned)stat_record->numpackets, (long long unsigned)bps, 
 			(long long unsigned)pps, (long long unsigned)bpp );
 	} else {
-		format_number(stat_record->numbytes, byte_str, VAR_LENGTH);
-		format_number(stat_record->numpackets, packet_str, VAR_LENGTH);
-		format_number(bps, bps_str, VAR_LENGTH);
-		format_number(pps, pps_str, VAR_LENGTH);
-		format_number(bpp, bpp_str, VAR_LENGTH);
+		format_number(stat_record->numbytes, byte_str, DONT_SCALE_NUMBER, VAR_LENGTH);
+		format_number(stat_record->numpackets, packet_str, DONT_SCALE_NUMBER, VAR_LENGTH);
+		format_number(bps, bps_str, DONT_SCALE_NUMBER, VAR_LENGTH);
+		format_number(pps, pps_str, DONT_SCALE_NUMBER, VAR_LENGTH);
+		format_number(bpp, bpp_str, DONT_SCALE_NUMBER, VAR_LENGTH);
 		printf("Summary: total flows: %llu, total bytes: %s, total packets: %s, avg bps: %s, avg pps: %s, avg bpp: %s\n",
 		(unsigned long long)stat_record->numflows, byte_str, packet_str, bps_str, pps_str, bpp_str );
 	}
@@ -491,7 +510,7 @@ int	v1_map_done = 0;
 				map->extension_size += extension_descriptor[EX_IO_SNMP_2].size;
 				map->extension_size += extension_descriptor[EX_AS_2].size;
 
-				if ( Insert_Extension_Map(&extension_map_list,map) && write_file ) {
+				if ( Insert_Extension_Map(extension_map_list,map) && write_file ) {
 					// flush new map
 					AppendToBuffer(nffile_w, (void *)map, map->size);
 				} // else map already known and flushed
@@ -530,6 +549,7 @@ int	v1_map_done = 0;
 		for ( i=0; i < nffile_r->block_header->NumRecords; i++ ) {
 
 			switch ( flow_record->type ) {
+				case CommonRecordV0Type:
 				case CommonRecordType:  {
 					int match;
 					uint32_t map_id = flow_record->ext_map;
@@ -538,16 +558,16 @@ int	v1_map_done = 0;
 						LogError("Corrupt data file. Extension map id %u too big.\n", flow_record->ext_map);
 						exit(255);
 					}
-					if ( extension_map_list.slot[map_id] == NULL ) {
+					if ( extension_map_list->slot[map_id] == NULL ) {
 						LogError("Corrupt data file. Missing extension map %u. Skip record.\n", flow_record->ext_map);
 						flow_record = (common_record_t *)((pointer_addr_t)flow_record + flow_record->size);	
 						continue;
 					} 
 
 					total_flows++;
-					master_record = &(extension_map_list.slot[map_id]->master_record);
+					master_record = &(extension_map_list->slot[map_id]->master_record);
 					Engine->nfrecord = (uint64_t *)master_record;
-					ExpandRecord_v2( flow_record, extension_map_list.slot[map_id], 
+					ExpandRecord_v2( flow_record, extension_map_list->slot[map_id], 
 						exp_info ? &(exp_info->info) : NULL, master_record);
 
 					// Time based filter
@@ -571,17 +591,17 @@ int	v1_map_done = 0;
 					UpdateStat(&stat_record, master_record);
 
 					// update number of flows matching a given map
-					extension_map_list.slot[map_id]->ref_count++;
+					extension_map_list->slot[map_id]->ref_count++;
 	
 					if ( flow_stat ) {
-						AddFlow(flow_record, master_record);
+						AddFlow(flow_record, master_record, extension_map_list->slot[map_id]);
 						if ( element_stat ) {
 							AddStat(flow_record, master_record);
 						} 
 					} else if ( element_stat ) {
 						AddStat(flow_record, master_record);
 					} else if ( sort_flows ) {
-						InsertFlow(flow_record, master_record);
+						InsertFlow(flow_record, master_record, extension_map_list->slot[map_id]);
 					} else {
 						if ( write_file ) {
 							AppendToBuffer(nffile_w, (void *)flow_record, flow_record->size);
@@ -608,7 +628,7 @@ int	v1_map_done = 0;
 				case ExtensionMapType: {
 					extension_map_t *map = (extension_map_t *)flow_record;
 	
-					if ( Insert_Extension_Map(&extension_map_list, map) && write_file ) {
+					if ( Insert_Extension_Map(extension_map_list, map) && write_file ) {
 						// flush new map
 						AppendToBuffer(nffile_w, (void *)map, map->size);
 					} // else map already known and flushed
@@ -681,7 +701,7 @@ int	v1_map_done = 0;
 		} // else stdout
 	}	 
 
-	PackExtensionMapList(&extension_map_list);
+	PackExtensionMapList(extension_map_list);
 
 	DisposeFile(nffile_r);
 	return stat_record;
@@ -702,9 +722,7 @@ int 		i, user_format, quiet, flow_stat, topN, aggregate, aggregate_mask, bidir;
 int 		print_stat, syntax_only, date_sorted, do_tag, compress, do_xstat;
 int			plain_numbers, GuessDir, pipe_output, csv_output;
 time_t 		t_start, t_end;
-uint16_t	Aggregate_Bits;
 uint32_t	limitflows;
-uint64_t	AggregateMasks[AGGR_SIZE];
 char 		Ident[IDENTLEN];
 
 	rfile = Rfile = Mdirs = wfile = ffile = filter = tstring = stat_type = NULL;
@@ -743,12 +761,9 @@ char 		Ident[IDENTLEN];
 	UnCompress_file	= NULL;
 	aggr_fmt		= NULL;
 	record_header 	= NULL;
-	Aggregate_Bits	= 0xFFFF;	// set all bits
 
 	Ident[0] = '\0';
 
-	for ( i=0; i<AGGR_SIZE; AggregateMasks[i++] = 0 ) ;
-
 	while ((c = getopt(argc, argv, "6aA:Bbc:D:E:s:hHn:i:j:f:qzr:v:w:K:M:NImO:R:XZt:TVv:x:l:L:o:")) != EOF) {
 		switch (c) {
 			case 'h':
@@ -813,10 +828,16 @@ char 		Ident[IDENTLEN];
                     exit(255);
                 } 
 				break;
-			case 'V':
-				printf("%s: Version: %s\n",argv[0], nfdump_version);
+			case 'V': {
+				char *e1, *e2;
+				e1 = "";
+				e2 = "";
+#ifdef NSEL
+				e1 = "NSEL-NEL";
+#endif
+				printf("%s: Version: %s%s%s\n",argv[0], e1, e2, nfdump_version);
 				exit(0);
-				break;
+				} break;
 			case 'l':
 				packet_limit_string = optarg;
 				break;
@@ -900,7 +921,7 @@ char 		Ident[IDENTLEN];
 				break;
 			case 'x':
 				query_file = optarg;
-				InitExtensionMaps(NULL);
+				InitExtensionMaps(NO_EXTENSION_LIST);
 				DumpExMaps(query_file);
 				exit(0);
 				break;
@@ -950,7 +971,7 @@ char 		Ident[IDENTLEN];
 		exit(255);
 	}
 
-	InitExtensionMaps(&extension_map_list);
+	extension_map_list = InitExtensionMaps(NEEDS_EXTENSION_LIST);
 	if ( !InitExporterList() ) {
 		exit(255);
 	}
@@ -1161,7 +1182,7 @@ char 		Ident[IDENTLEN];
 			nffile_t *nffile = OpenNewFile(wfile, NULL, compress, is_anonymized, NULL);
 			if ( !nffile ) 
 				exit(255);
-			if ( ExportFlowTable(nffile, aggregate, bidir, date_sorted) ) {
+			if ( ExportFlowTable(nffile, aggregate, bidir, date_sorted, extension_map_list) ) {
 				CloseUpdateFile(nffile, Ident );	
 			} else {
 				CloseFile(nffile);
@@ -1169,19 +1190,19 @@ char 		Ident[IDENTLEN];
 			}
 			DisposeFile(nffile);
 		} else {
-			PrintFlowTable(print_record, limitflows, do_tag, GuessDir);
+			PrintFlowTable(print_record, limitflows, do_tag, GuessDir, extension_map_list);
 		}
 	}
 
 	if (flow_stat) {
-		PrintFlowStat(record_header, print_record, topN, do_tag, quiet, csv_output);
+		PrintFlowStat(record_header, print_record, topN, do_tag, quiet, csv_output, extension_map_list);
 #ifdef DEVEL
 		printf("Loopcnt: %u\n", loopcnt);
 #endif
 	} 
 
 	if (element_stat) {
-		PrintElementStat(&sum_stat, record_header, print_record, topN, do_tag, quiet, pipe_output, csv_output);
+		PrintElementStat(&sum_stat, plain_numbers, record_header, print_record, topN, do_tag, quiet, pipe_output, csv_output);
 	} 
 
 	if ( !quiet ) {
@@ -1205,7 +1226,7 @@ char 		Ident[IDENTLEN];
 
 	Dispose_FlowTable();
 	Dispose_StatTable();
-	FreeExtensionMaps(&extension_map_list);
+	FreeExtensionMaps(extension_map_list);
 
 #ifdef DEVEL
 	if ( hash_hit || hash_miss )
