@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2011, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +12,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -38,6 +39,9 @@
 #include "config.h"
 
 #include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 #include <sys/uio.h>
 #include <unistd.h>
 #include <string.h>
@@ -81,6 +85,8 @@ static char	error_string[ERR_SIZE];
 
 static int LZO_initialize(void);
 
+static int OpenRaw(char *filename, stat_record_t *stat_record, int *compressed);
+
 extern char *nf_error;
 
 /* function prototypes */
@@ -250,7 +256,7 @@ void CloseFile(nffile_t *nffile){
 int ChangeIdent(char *filename, char *Ident) {
 file_header_t	FileHeader;
 struct stat stat_buf;
-int fd, ret;
+int fd;
 
 	if ( filename == NULL ) 
 		return 0;
@@ -271,7 +277,11 @@ int fd, ret;
 		return fd;
 	}
 
-	ret = read(fd, (void *)&FileHeader, sizeof(FileHeader));
+	if ( read(fd, (void *)&FileHeader, sizeof(FileHeader)) < 0 ) {
+		LogError("read() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		close(fd);
+		return -1;
+	}
 	if ( FileHeader.magic != MAGIC ) {
 		LogError("Open file '%s': bad magic: 0x%X\n", filename, FileHeader.magic );
 		close(fd);
@@ -347,6 +357,7 @@ nffile_t *nffile;
 	}
 	nffile->buff_ptr = NULL;
 	nffile->fd	 	= 0;
+	nffile->catalog = NULL;
 
 	// Init file header
 	nffile->file_header = calloc(1, sizeof(file_header_t));
@@ -365,6 +376,19 @@ nffile_t *nffile;
 		return NULL;
 	}
 
+/*
+	XXX catalogs not yet implemented
+	nffile->catalog = calloc(1, sizeof(catalog_t));
+	if ( !nffile->catalog ) {
+		LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		return NULL;
+	}
+	nffile->catalog->NumRecords = 0;
+	nffile->catalog->size 		= sizeof(catalog_t) - sizeof(data_block_header_t);
+	nffile->catalog->id 		= CATALOG_BLOCK;
+	nffile->catalog->pad 		= 0;
+	nffile->catalog->reserved 	= 0;
+*/
 	// init data buffer
 	nffile->block_header = malloc(BUFFSIZE + sizeof(data_block_header_t));
 	if ( !nffile->block_header ) {
@@ -374,7 +398,7 @@ nffile_t *nffile;
 	nffile->block_header->size 		 = 0;
 	nffile->block_header->NumRecords = 0;
 	nffile->block_header->id		 = DATA_BLOCK_TYPE_2;
-	nffile->block_header->pad		 = 0;
+	nffile->block_header->flags		 = 0;
 
 	nffile->buff_ptr = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t));
 	
@@ -419,9 +443,19 @@ int 			flags;
 		}
 	}
 
-	memset((void *)nffile->stat_record, 0, sizeof(stat_record_t));
-	nffile->stat_record->first_seen = 0x7fffffff;
-	nffile->stat_record->msec_first = 999;
+/*
+	XXX catalogs not yet implemented
+	if ( nffile->catalog && nffile->catalog->NumRecords ) {
+		memset((void *)nffile->catalog->entries, 0, nffile->catalog->NumRecords * sizeof(struct catalog_entry_s));
+		nffile->catalog->NumRecords = 0;
+		nffile->catalog->size		= 0;
+	} 
+*/
+	if ( nffile->stat_record ) {
+		memset((void *)nffile->stat_record, 0, sizeof(stat_record_t));
+		nffile->stat_record->first_seen = 0x7fffffff;
+		nffile->stat_record->msec_first = 999;
+	}
 
 	if ( ident ) {
 		strncpy(nffile->file_header->ident, ident, IDENTLEN);
@@ -453,6 +487,15 @@ int 			flags;
 		return NULL;
 	}
 
+/* skip writing catalog in this test version
+	XXX catalogs not yet implemented
+	if ( WriteExtraBlock(nffile, (data_block_header_t *)nffile->catalog) < 0 ) {
+		LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		close(nffile->fd);
+		return NULL;
+	}
+*/
+
 	return nffile;
 
 } /* End of OpenNewFile */
@@ -485,7 +528,7 @@ nffile_t		*nffile;
 	nffile->block_header->size 		 = 0;
 	nffile->block_header->NumRecords = 0;
 	nffile->block_header->id		 = DATA_BLOCK_TYPE_2;
-	nffile->block_header->pad		 = 0;
+	nffile->block_header->flags		 = 0;
 	nffile->buff_ptr = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t));
 
 	// initialize output  lzo buffer
@@ -502,8 +545,152 @@ nffile_t		*nffile;
 
 } /* End of AppendFile */
 
+int RenameAppend(char *from, char *to) {
+int fd_to, fd_from, ret;
+int compressed_to, compressed_from;
+stat_record_t stat_record_to, stat_record_from;
+data_block_header_t *block_header;
+void *p;
+
+	fd_to = OpenRaw(to, &stat_record_to, &compressed_to);
+	if ( fd_to == 0 ) {
+		// file does not exists, use rename
+		return rename(from, to) == 0 ? 1 : 0;
+	}
+
+	fd_from = OpenRaw(from, &stat_record_from, &compressed_from);
+	if ( fd_from <= 0 ) {
+		// file does not exists - strange
+		close(fd_to);
+		return 0;
+	}
+
+	// both files open - append data
+	ret = lseek(fd_to, 0, SEEK_END);
+	if ( ret < 0 ) {
+		LogError("lseek() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		close(fd_from);
+		close(fd_to);
+		return 0;
+	}
+
+	block_header = malloc(sizeof(data_block_header_t) + BUFFSIZE);
+	if ( !block_header ) {
+		LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+		close(fd_from);
+		close(fd_to);
+		return 0;
+	}
+	p = (void *)((void *)block_header + sizeof(data_block_header_t));
+
+	while (1) {
+		ret = read(fd_from, (void *)block_header, sizeof(data_block_header_t));
+		if ( ret == 0 ) 
+			// EOF
+			break;
+
+		if ( ret < 0 ) {
+			// that's bad! difficult to recover. stat will be inconsistent
+			LogError("read() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+			break;
+		}
+
+		// read data block
+		ret = read(fd_from, p, block_header->size);
+		if ( ret != block_header->size ) {
+			// that's bad! difficult to recover. stat will be inconsistent
+			LogError("read() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+			break;
+		}
+		// append data block
+		ret = write(fd_to, block_header, sizeof(data_block_header_t) + block_header->size);
+		if ( ret < 0 ) {
+			// that's bad! difficult to recover. stat will be inconsistent
+			LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+			break;
+		}
+	}
+
+	SumStatRecords(&stat_record_to, &stat_record_from);
+	// both files open - append data
+	ret = lseek(fd_to, sizeof(file_header_t), SEEK_SET);
+	if ( ret < 0 ) {
+		LogError("lseek() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		close(fd_from);
+		close(fd_to);
+		return 0;
+	}
+
+	if ( write(fd_to, (void *)&stat_record_to, sizeof(stat_record_t)) <= 0 ) {
+		LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		close(fd_from);
+		close(fd_to);
+		return 0;
+	}
+
+	close(fd_from);
+	close(fd_to);
+	unlink(from);
+	return 1;
+
+} // End of RenameAppend
+
+static int OpenRaw(char *filename, stat_record_t *stat_record, int *compressed) {
+struct stat stat_buf;
+file_header_t file_header;
+int fd, ret;
+
+	if ( stat(filename, &stat_buf) ) {
+		// file does not exists
+		return 0;
+	}
+
+	// file exists - should be a regular file 
+	if (!S_ISREG(stat_buf.st_mode) ) {
+		// should nor really happen - catch it anyway
+		LogError("'%s' is not a regular file\n", filename);
+		return -1;
+	}
+
+	// file exists - append to existing
+	fd = open(filename, O_RDWR, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH );
+	if ( fd < 0 ) {
+		LogError("open() failed for file %s: '%s'" , filename, strerror(errno));
+		return -1;
+	}
+
+	ret = read(fd, (void *)&file_header, sizeof(file_header_t));
+	if ( ret < 0 ) {
+		LogError("read() failed for file %s: '%s'" , filename, strerror(errno));
+		close(fd);
+		return -1;
+	}
+
+	if ( file_header.magic != MAGIC ) {
+		LogError("Open file '%s': bad magic: 0x%X\n", filename, file_header.magic );
+		close(fd);
+		return -1;
+	}
+
+	if ( file_header.version != LAYOUT_VERSION_1 ) {
+		LogError("Open file %s: bad version: %u\n", filename, file_header.version );
+		close(fd);
+		return -1;
+	}
+
+	ret = read(fd, (void *)stat_record, sizeof(stat_record_t));
+	if ( ret < 0 ) {
+		LogError("read() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		close(fd);
+		return -1;
+	}
+	
+	*compressed = file_header.flags & FLAG_COMPRESSED ? 1 : 0;
+	return fd;
+
+} // End of OpenRaw
+
 int CloseUpdateFile(nffile_t *nffile, char *ident) {
-file_header_t	file_header;
 
 	if ( nffile->block_header->size ) {
 		int ret = WriteBlock(nffile);
@@ -532,7 +719,6 @@ file_header_t	file_header;
 		if ( strlen(nffile->file_header->ident) == 0 ) 
 		strncpy(nffile->file_header->ident, IDENTNONE, IDENTLEN);
 	}
-	file_header.ident[IDENTLEN - 1] = 0;
 
 	if ( write(nffile->fd, (void *)nffile->file_header, sizeof(file_header_t)) <= 0 ) {
 		LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
@@ -620,10 +806,11 @@ void 	*read_ptr, *buff;
 	read_ptr 	 = (void *)((pointer_addr_t)buff + buff_bytes);	
 	do {
 		ret = read(nffile->fd, read_ptr, request_size);
-		if ( ret < 0 ) 
+		if ( ret < 0 ) {
 			// -1: Error - not expected
 			LogError("read() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 			return NF_ERROR;
+		}
 
 		if ( ret == 0 ) {
 			//  0: EOF   - not expected
@@ -719,7 +906,7 @@ unsigned char __LZO_MMODEL *out;
 lzo_uint in_len;
 lzo_uint out_len;
 
-	if ( !TestFlag(nffile->file_header->flags, FLAG_COMPRESSED) ) {
+	if ( !TestFlag(nffile->file_header->flags, FLAG_COMPRESSED) || block_header->id == CATALOG_BLOCK  ) {
 		ret =  write(nffile->fd, (void *)block_header, sizeof(data_block_header_t) + block_header->size);
 		if ( ret > 0 ) {
 			nffile->file_header->NumBlocks++;
@@ -764,7 +951,9 @@ void		*p = (void *)input_record;
 
 	if ( (input_record->flags & FLAG_IPV6_ADDR) != 0 )	{ // IPv6
 		// IPv6
-		memcpy((void *)output_record->v6.srcaddr, p, 4 * sizeof(uint64_t));	
+		// keep compiler happy
+		// memcpy((void *)output_record->v6.srcaddr, p, 4 * sizeof(uint64_t));	
+		memcpy((void *)output_record->ip_union._ip_64.addr, p, 4 * sizeof(uint64_t));	
 		p = (void *)((pointer_addr_t)p + 4 * sizeof(uint64_t));
 	} else { 	
 		// IPv4
@@ -810,7 +999,7 @@ void		*p = (void *)input_record;
 } // End of ExpandRecord_v1
 
 void UnCompressFile(char * filename) {
-int 			i, flags, compressed, anonymized;
+int 			i, compressed, anonymized;
 ssize_t			ret;
 nffile_t		*nffile_r, *nffile_w;
 stat_record_t	*_s;
@@ -826,7 +1015,6 @@ void			*tmp;
 	snprintf(outfile, MAXPATHLEN, "%s-tmp", filename);
 	outfile[MAXPATHLEN-1] = '\0';
 
-	flags = nffile_r->file_header->flags;
 	if ( FILE_IS_COMPRESSED(nffile_r) ) {
 		printf("Uncompress file %s ..\n", filename);
 		compressed = 0;
@@ -1033,6 +1221,8 @@ int fd, ret;
 
 } // End of GetStatRecord
 
+
+
 #ifdef COMPAT15
 /*
  * v1 -> v2 record conversion:
@@ -1048,7 +1238,7 @@ int fd, ret;
  *												uint16_t	size;
  *
  *  1 uint16_t    size;							uint8_t		flags;		
- * 												uint8_t 	exporter_ref;
+ * 												uint8_t 	exporter_sysid;
  *    uint16_t    exporter_ref; => 0			uint16_t	ext_map;
  *
  *  2 uint16_t    msec_first;					uint16_t	msec_first;
@@ -1092,9 +1282,9 @@ size_t cplen;
 	v2->size = v1->size;
 
 	// index 1
-	v2->flags 		 = tmp1;
-	v2->exporter_ref = 0;
-	v2->ext_map 	 = 0;
+	v2->flags 		   = tmp1;
+	v2->exporter_sysid = 0;
+	v2->ext_map 	   = 0;
 
 	// index 2, 3, 4 already in sync
 
