@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +12,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -62,17 +63,27 @@
 #include "nfx.h"
 #include "nfstat.h"
 #include "nfstatfile.h"
+#include "bookkeeper.h"
 #include "nfxstat.h"
+#include "collector.h"
+#include "exporter.h"
 #include "ipconv.h"
 #include "flist.h"
 #include "util.h"
 #include "profile.h"
 
+/* externals */
+extern generic_exporter_t **exporter_list;
+
+#ifdef COMPAT15
+extern extension_descriptor_t extension_descriptor[];
+#endif
+
 /* Local Variables */
 static const char *nfdump_version = VERSION;
 
 
-extension_map_list_t extension_map_list;
+extension_map_list_t *extension_map_list;
 uint32_t is_anonymized;
 char Ident[IDENTLEN];
 
@@ -110,7 +121,6 @@ static void usage(char *name) {
 
 static void process_data(profile_channel_info_t *channels, unsigned int num_channels, time_t tslot, int do_xstat) {
 common_record_t	*flow_record;
-master_record_t	master_record;
 nffile_t		*nffile;
 FilterEngine_data_t	*engine;
 int 		i, j, done, ret ;
@@ -135,10 +145,6 @@ int	v1_map_done = 0;
 	strncpy(Ident, nffile->file_header->ident, IDENTLEN);
 	Ident[IDENTLEN-1] = '\0';
 
-	for ( j=0; j < num_channels; j++ ) {
-		(channels[j].engine)->nfrecord 		= (uint64_t *)&master_record;
-	}
-
 	done = 0;
 	while ( !done ) {
 
@@ -174,17 +180,25 @@ int	v1_map_done = 0;
 			if ( v1_map_done == 0 ) {
 				extension_map_t *map = malloc(sizeof(extension_map_t) + 2 * sizeof(uint16_t) );
 				if ( ! map ) {
-					perror("Memory allocation error");
+					LogError("malloc() allocation error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 					exit(255);
 				}
 				map->type 	= ExtensionMapType;
 				map->size 	= sizeof(extension_map_t) + 2 * sizeof(uint16_t);
+				if (( map->size & 0x3 ) != 0 ) {
+					map->size += 4 - ( map->size & 0x3 );
+				}
+				
 				map->map_id = INIT_ID;
 				map->ex_id[0]  = EX_IO_SNMP_2;
 				map->ex_id[1]  = EX_AS_2;
 				map->ex_id[2]  = 0;
 
-				if ( Insert_Extension_Map(&extension_map_list, map) ) {
+				map->extension_size  = 0;
+				map->extension_size += extension_descriptor[EX_IO_SNMP_2].size;
+				map->extension_size += extension_descriptor[EX_AS_2].size;
+				
+				if ( Insert_Extension_Map(extension_map_list, map) ) {
 					int j;
 					for ( j=0; j < num_channels; j++ ) {
 						if ( channels[j].nffile != NULL) {
@@ -220,62 +234,106 @@ int	v1_map_done = 0;
 
 		flow_record = nffile->buff_ptr;
 		for ( i=0; i < nffile->block_header->NumRecords; i++ ) {
-			if ( flow_record->type == CommonRecordType ) {
-				if ( extension_map_list.slot[flow_record->ext_map] == NULL ) {
-					LogError("Corrupt data file. Missing extension map %u. Skip record.\n", flow_record->ext_map);
-					flow_record = (common_record_t *)((pointer_addr_t)flow_record + flow_record->size);	
-					continue;
-				} 
+			switch ( flow_record->type ) { 
+					case CommonRecordV0Type:
+					case CommonRecordType: {
+					generic_exporter_t *exp_info = exporter_list[flow_record->exporter_sysid];
+					uint32_t map_id = flow_record->ext_map;
+					master_record_t	*master_record;
 
-				ExpandRecord_v2( flow_record, extension_map_list.slot[flow_record->ext_map], &master_record);
-
-				for ( j=0; j < num_channels; j++ ) {
-					int match;
-
-					// apply profile filter
-					engine = channels[j].engine;
-					match = (*engine->FilterEngine)(engine);
-
-					// if profile filter failed -> next profile
-					if ( !match )
+					if ( extension_map_list->slot[map_id] == NULL ) {
+						LogError("Corrupt data file. Missing extension map %u. Skip record.\n", flow_record->ext_map);
+						flow_record = (common_record_t *)((pointer_addr_t)flow_record + flow_record->size);	
 						continue;
-
-					// filter was successful -> continue record processing
-
-					// update statistics
-					UpdateStat(&channels[j].stat_record, &master_record);
-					if ( channels[j].nffile ) 
-						UpdateStat(channels[j].nffile->stat_record, &master_record);
-
-					if ( channels[j].xstat ) 
-						UpdateXStat(channels[j].xstat, &master_record);
-
-					// do we need to write data to new file - shadow profiles do not have files.
-					// check if we need to flush the output buffer
-					if ( channels[j].nffile != NULL ) {
-						// write record to output buffer
-						AppendToBuffer(channels[j].nffile, (void *)flow_record, flow_record->size);
 					} 
+	
+					master_record = &(extension_map_list->slot[map_id]->master_record);
+					ExpandRecord_v2( flow_record, extension_map_list->slot[flow_record->ext_map], 
+						exp_info ? &(exp_info->info) : NULL, master_record);
 
-				} // End of for all channels
-
-			} else if ( flow_record->type == ExtensionMapType ) {
-				extension_map_t *map = (extension_map_t *)flow_record;
-
-				if ( Insert_Extension_Map(&extension_map_list, map) ) {
-					int j;
 					for ( j=0; j < num_channels; j++ ) {
+						int match;
+	
+						// apply profile filter
+						(channels[j].engine)->nfrecord 	= (uint64_t *)master_record;
+						engine = channels[j].engine;
+						match = (*engine->FilterEngine)(engine);
+	
+						// if profile filter failed -> next profile
+						if ( !match )
+							continue;
+	
+						// filter was successful -> continue record processing
+	
+						// update statistics
+						UpdateStat(&channels[j].stat_record, master_record);
+						if ( channels[j].nffile ) 
+							UpdateStat(channels[j].nffile->stat_record, master_record);
+	
+						if ( channels[j].xstat ) 
+							UpdateXStat(channels[j].xstat, master_record);
+	
+						// do we need to write data to new file - shadow profiles do not have files.
+						// check if we need to flush the output buffer
 						if ( channels[j].nffile != NULL ) {
-							// flush new map
-							AppendToBuffer(channels[j].nffile, (void *)map, map->size);
+							// write record to output buffer
+							AppendToBuffer(channels[j].nffile, (void *)flow_record, flow_record->size);
+						} 
+	
+					} // End of for all channels
+	
+					} break;
+				case ExtensionMapType: {
+					extension_map_t *map = (extension_map_t *)flow_record;
+	
+					if ( Insert_Extension_Map(extension_map_list, map) ) {
+						int j;
+						for ( j=0; j < num_channels; j++ ) {
+							if ( channels[j].nffile != NULL ) {
+								// flush new map
+								AppendToBuffer(channels[j].nffile, (void *)map, map->size);
+							}
 						}
+					} // else map already known and flushed
+	
+					} break; 
+				case ExporterInfoRecordType: {
+					int ret = AddExporterInfo((exporter_info_record_t *)flow_record);
+					if ( ret != 0 ) {
+						int j;
+						for ( j=0; j < num_channels; j++ ) {
+							if ( channels[j].nffile != NULL && ret == 1) {
+								// flush new exporter
+								AppendToBuffer(channels[j].nffile, (void *)flow_record, flow_record->size);
+							}
+						}
+					} else {
+						LogError("Failed to add Exporter Record\n");
 					}
-				} // else map already known and flushed
-
-			} else {
-				LogError("Skip unknown record type %i\n", flow_record->type);
+					} break;
+				case SamplerInfoRecordype: {
+					int ret = AddSamplerInfo((sampler_info_record_t *)flow_record);
+					if ( ret != 0 ) {
+						int j;
+						for ( j=0; j < num_channels; j++ ) {
+							if ( channels[j].nffile != NULL && ret == 1 ) {
+								// flush new map
+								AppendToBuffer(channels[j].nffile, (void *)flow_record, flow_record->size);
+							}
+						}
+					} else {
+						LogError("Failed to add Sampler Record\n");
+					}
+					} break;
+				case ExporterRecordType:
+				case SamplerRecordype:
+				case ExporterStatRecordType:
+						// Silently skip exporter records
+					break;
+				default:  {
+					LogError("Skip unknown record type %i\n", flow_record->type);
+				}
 			}
-
 			// Advance pointer by number of bytes for netflow record
 			flow_record = (common_record_t *)((pointer_addr_t)flow_record + flow_record->size);
 
@@ -619,7 +677,10 @@ time_t tslot;
 		exit(255);
 	}
 
-	InitExtensionMaps(&extension_map_list);
+	extension_map_list = InitExtensionMaps(NEEDS_EXTENSION_LIST);
+	if ( !InitExporterList() ) {
+		exit(255);
+	}
 
 	SetupInputFileSequence(Mdirs,rfile, NULL);
 
@@ -627,5 +688,7 @@ time_t tslot;
 
 	CloseChannels(tslot, compress);
 
+	FreeExtensionMaps(extension_map_list);
+
 	return 0;
 }
