@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +12,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -68,6 +69,7 @@
 #include "bookkeeper.h"
 #include "nfxstat.h"
 #include "collector.h"
+#include "exporter.h"
 #include "netflow_v5_v7.h"
 #include "netflow_v9.h"
 #include "nfprof.h"
@@ -89,6 +91,10 @@
 /* Externals */
 extern int yydebug;
 
+#ifdef COMPAT15
+extern extension_descriptor_t extension_descriptor[];
+#endif
+
 /* Global Variables */
 FilterEngine_data_t	*Engine;
 int 		verbose;
@@ -98,7 +104,9 @@ static const char *nfdump_version = VERSION;
 
 send_peer_t peer;
 
-extension_map_list_t extension_map_list;
+extension_map_list_t *extension_map_list;
+
+generic_exporter_t **exporter_list;
 
 /* Function Prototypes */
 static void usage(char *name);
@@ -286,12 +294,19 @@ int	v1_map_done = 0;
 				}
 				map->type 	= ExtensionMapType;
 				map->size 	= sizeof(extension_map_t) + 2 * sizeof(uint16_t);
+				if (( map->size & 0x3 ) != 0 ) {
+					map->size += 4 - ( map->size & 0x3 );
+				}
 				map->map_id = INIT_ID;
 				map->ex_id[0]  = EX_IO_SNMP_2;
 				map->ex_id[1]  = EX_AS_2;
 				map->ex_id[2]  = 0;
 				
-				Insert_Extension_Map(&extension_map_list, map);
+				map->extension_size  = 0;
+				map->extension_size += extension_descriptor[EX_IO_SNMP_2].size;
+				map->extension_size += extension_descriptor[EX_AS_2].size;
+					
+				Insert_Extension_Map(extension_map_list, map);
 				v1_map_done = 1;
 			}
 
@@ -318,75 +333,86 @@ int	v1_map_done = 0;
 		for ( i=0; i < nffile->block_header->NumRecords; i++ ) {
 			int match;
 
-			if ( flow_record->type == CommonRecordType ) {
-				if ( extension_map_list.slot[flow_record->ext_map] == NULL ) {
-					LogError("Corrupt data file. Missing extension map %u. Skip record.\n", flow_record->ext_map);
-					flow_record = (common_record_t *)((pointer_addr_t)flow_record + flow_record->size);	
-					continue;
-				} 
+			switch ( flow_record->type ) {
+				case CommonRecordV0Type:
+				case CommonRecordType: {
+					if ( extension_map_list->slot[flow_record->ext_map] == NULL ) {
+						LogError("Corrupt data file. Missing extension map %u. Skip record.\n", flow_record->ext_map);
+						flow_record = (common_record_t *)((pointer_addr_t)flow_record + flow_record->size);	
+						continue;
+					} 
 
-				// if no filter is given, the result is always true
-				ExpandRecord_v2( flow_record, extension_map_list.slot[flow_record->ext_map], &master_record);
+					// if no filter is given, the result is always true
+					ExpandRecord_v2( flow_record, extension_map_list->slot[flow_record->ext_map], NULL, &master_record);
 
-				match = twin_start && (master_record.first < twin_start || master_record.last > twin_end) ? 0 : 1;
+					match = twin_start && (master_record.first < twin_start || master_record.last > twin_end) ? 0 : 1;
 
-				// filter netflow record with user supplied filter
-				if ( match ) 
-					match = (*Engine->FilterEngine)(Engine);
-
-				if ( match == 0 ) { // record failed to pass all filters
-					// increment pointer by number of bytes for netflow record
-					flow_record = (common_record_t *)((pointer_addr_t)flow_record + flow_record->size);	
-					// go to next record
-					continue;
-				}
-				// Records passed filter -> continue record processing
-
-				if ( netflow_version == 5 ) 
-					again = Add_v5_output_record(&master_record, &peer);
-				else
-					again = Add_v9_output_record(&master_record, &peer);
-
-				cnt++;
-				numflows++;
-
-				if ( peer.flush ) {
-					ret = FlushBuffer(confirm);
+					// filter netflow record with user supplied filter
+					if ( match ) 
+						match = (*Engine->FilterEngine)(Engine);
 	
-					if ( ret < 0 ) {
-						perror("Error sending data");
-						CloseFile(nffile);
-						DisposeFile(nffile);
-						return;
+					if ( match == 0 ) { // record failed to pass all filters
+						// increment pointer by number of bytes for netflow record
+						flow_record = (common_record_t *)((pointer_addr_t)flow_record + flow_record->size);	
+						// go to next record
+						continue;
 					}
-		
-					if ( delay ) {
-						// sleep as specified
-						usleep(delay);
-					}
-					cnt = 0;
-				}
+					// Records passed filter -> continue record processing
 
-				if ( again ) {
 					if ( netflow_version == 5 ) 
-						Add_v5_output_record(&master_record, &peer);
+						again = Add_v5_output_record(&master_record, &peer);
 					else
-						Add_v9_output_record(&master_record, &peer);
+						again = Add_v9_output_record(&master_record, &peer);
+	
 					cnt++;
+					numflows++;
+
+					if ( peer.flush ) {
+						ret = FlushBuffer(confirm);
+	
+						if ( ret < 0 ) {
+							perror("Error sending data");
+							CloseFile(nffile);
+							DisposeFile(nffile);
+							return;
+						}
+			
+						if ( delay ) {
+							// sleep as specified
+							usleep(delay);
+						}
+						cnt = 0;
+					}
+	
+					if ( again ) {
+						if ( netflow_version == 5 ) 
+							Add_v5_output_record(&master_record, &peer);
+						else
+							Add_v9_output_record(&master_record, &peer);
+						cnt++;
+					}
+
+					} break;
+				case ExtensionMapType: {
+					extension_map_t *map = (extension_map_t *)flow_record;
+	
+					if ( Insert_Extension_Map(extension_map_list, map) ) {
+						// flush new map
+						
+					} // else map already known and flushed
+	
+					} break;
+				case ExporterRecordType:
+				case SamplerRecordype:
+				case ExporterInfoRecordType:
+				case ExporterStatRecordType:
+				case SamplerInfoRecordype:
+						// Silently skip exporter/sampler records
+					break;
+			 	default: {
+					LogError("Skip unknown record type %i\n", flow_record->type);
 				}
-
-			} else if ( flow_record->type == ExtensionMapType ) {
-				extension_map_t *map = (extension_map_t *)flow_record;
-
-				if ( Insert_Extension_Map(&extension_map_list, map) ) {
-					// flush new map
-					
-				} // else map already known and flushed
-
-			} else {
-				LogError("Skip unknown record type %i\n", flow_record->type);
 			}
-
 			// Advance pointer by number of bytes for netflow record
 			flow_record = (common_record_t *)((pointer_addr_t)flow_record + flow_record->size);	
 
@@ -582,7 +608,7 @@ time_t t_start, t_end;
 		exit(0);
 	}
 
-	InitExtensionMaps(&extension_map_list);
+	extension_map_list = InitExtensionMaps(NEEDS_EXTENSION_LIST);
 
 	SetupInputFileSequence(NULL,rfile, NULL);
 
@@ -593,5 +619,7 @@ time_t t_start, t_end;
 
 	send_data(rfile, t_start, t_end, count, delay, confirm, netflow_version);
 
+	FreeExtensionMaps(extension_map_list);
+
 	return 0;
 }
