@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +12,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -47,11 +48,19 @@
  * are stored into an internal hash table.
  */
 
+typedef struct SumRecord_s {
+	uint64_t	flows;
+	uint64_t	ipkg;
+	uint64_t	opkg;
+	uint64_t	ibyte;
+	uint64_t	obyte;
+} SumRecord_t;
+
 typedef struct StatRecord {
 	// record chain
 	struct StatRecord *next;
 	// flow parameters
-	uint64_t	counter[3];
+	uint64_t	counter[5];	// flows ipkg ibyte opkg obyte
 	uint32_t	first;
 	uint32_t	last;
 	uint16_t	msec_first;
@@ -87,6 +96,9 @@ typedef struct SortElement {
     uint64_t	count;
 } SortElement_t;
 
+#define MULTIPLE_LIST_ORDERS 1
+#define SINGLE_LIST_ORDER    0
+
 /* Function prototypes */
 void SetLimits(int stat, char *packet_limit_string, char *byte_limit_string );
 
@@ -96,17 +108,17 @@ void Dispose_StatTable(void);
 
 int SetStat(char *str, int *element_stat, int *flow_stat);
 
-int SetStat_DefaultOrder(char *order);
-
-void InsertFlow(common_record_t *raw_record, master_record_t *flow_record);
+int Parse_PrintOrder(char *order);
 
 void AddStat(common_record_t *raw_record, master_record_t *flow_record );
 
-void PrintFlowTable(printer_t print_record, uint32_t limitflows, int date_sorted, int tag, int GuessDir);
+void PrintFlowTable(printer_t print_record, uint32_t limitflows, int tag, int GuessDir, extension_map_list_t *extension_map_list);
 
-void PrintFlowStat(char *record_header, printer_t print_record, int topN, int tag, int quiet, int cvs_output);
+void PrintFlowStat(char *record_header, printer_t print_record, int topN, int tag, int quiet, int cvs_output, extension_map_list_t *extension_map_list);
 
-void PrintElementStat(stat_record_t	*sum_stat, char *record_header, printer_t print_record, int topN, int tag, int quiet, int pipe_output, int cvs_output);
+void PrintElementStat(stat_record_t	*sum_stat, uint32_t limitflows, char *record_header, printer_t print_record, int topN, int tag, int quiet, int pipe_output, int cvs_output);
+
+int ParseListOrder(char *s, int multiple_orders );
 
 void PrintSortedFlows(printer_t print_record, uint32_t limitflows, int tag);
 
