@@ -1,4 +1,5 @@
 /*
+ *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
@@ -11,7 +12,7 @@
  *   * Redistributions in binary form must reproduce the above copyright notice, 
  *     this list of conditions and the following disclaimer in the documentation 
  *     and/or other materials provided with the distribution.
- *   * Neither the name of SWITCH nor the names of its contributors may be 
+ *   * Neither the name of the author nor the names of its contributors may be 
  *     used to endorse or promote products derived from this software without 
  *     specific prior written permission.
  *  
@@ -468,7 +469,7 @@ int	evaluate, invert;
 				break;
 			case CMP_IDENT:
 				value = args->filter[index].value;
-				evaluate = strncmp(CurrentIdent, args->IdentList[value], IdentLen) == 0 ;
+				evaluate = strncmp(CurrentIdent, args->IdentList[value], IDENTLEN) == 0 ;
 				break;
 			case CMP_FLAGS:
 				if ( invert )
